#include "system.h"			// system configuration file
#include "Queue.h"			// UART command parser
#include <cplb.h>



// use cache
// DATA_BANK A, upper half is data cache
// DATA_BANK A, lower half is L1 memory
// DATA_BANK B  is L1 memory
// NO intruction cache, all code is in L1
// uses customised "cplbtab533.s" file to exclude ASYNC memory banks from being cached
int __cplb_ctrl = CPLB_ENABLE_DCPLBS | CPLB_ENABLE_ICPLBS | CPLB_ENABLE_DCACHE;


// Declare the DMA buffers		each for 2*Number_of_Samples  samples of both input (RIN, SIN) or output (ROUT< SOUT) channels
// Order in buffer is: 	

//	Receive Buffer:
//		RIN0(0).. RIN31(0), RIN0(1)..RIN31(1) ... ... RIN0(Number_of_Samples-1)..RIN31(Number_of_Samples-1) ....
//		SIN0(0).. SIN31(0), SIN0(1)..SIN31(1) ... ... SIN0(Number_of_Samples-1)..SIN31(Number_of_Samples-1)
//	Transmit Buffer:
//		ROUT0(0).. ROUT31(0), ROUT0(1)..ROUT31(1) ... ... ROUT0(Number_of_Samples-1)..ROUT31(Number_of_Samples-1) ....
//		SOUT0(0).. SOUT31(0), SOUT0(1)..SOUT31(1) ... ... SOUT0(Number_of_Samples-1)..SOUT31(Number_of_Samples-1)

section("DMA_Buffers0") volatile short sSport0RxBuffer[Number_of_Channels * Number_of_Samples * 2 * 2];
section("DMA_Buffers1") volatile short sSport0TxBuffer[Number_of_Channels * Number_of_Samples * 2 * 2];


// set up DMA descriptors (1st half and second half, then repeat)
// small descriptor model, only start address needs to be fetched


tDMA_descriptor DMA_TX_second; // declaration
tDMA_descriptor DMA_RX_second; // declaration
section("L1_data_a") tDMA_descriptor DMA_TX_first = {&DMA_TX_second, sSport0TxBuffer};
section("L1_data_a") tDMA_descriptor DMA_TX_second = {&DMA_TX_first, (sSport0TxBuffer + sizeof(sSport0TxBuffer)/sizeof(sSport0TxBuffer[0])/2)};
section("L1_data_a") tDMA_descriptor DMA_RX_first = {&DMA_RX_second, sSport0RxBuffer};
section("L1_data_a") tDMA_descriptor DMA_RX_second = {&DMA_RX_first, (sSport0RxBuffer + sizeof(sSport0RxBuffer)/sizeof(sSport0RxBuffer[0])/2)};


volatile short* rin_current_half[2] = { (sSport0RxBuffer),
												   (sSport0RxBuffer + sizeof(sSport0RxBuffer)/sizeof(sSport0RxBuffer[0]) / 2)
												 };
volatile short* sin_current_half[2] = { (sSport0RxBuffer + sizeof(sSport0RxBuffer)/sizeof(sSport0RxBuffer[0]) / 4),
												   (sSport0RxBuffer + sizeof(sSport0RxBuffer)/sizeof(sSport0RxBuffer[0]) * 3 / 4)
												 };
volatile short* rout_current_half[2] = { (sSport0TxBuffer),
												   (sSport0TxBuffer + sizeof(sSport0TxBuffer)/sizeof(sSport0TxBuffer[0]) / 2)
												 };
volatile short* sout_current_half[2] = { (sSport0TxBuffer + sizeof(sSport0TxBuffer)/sizeof(sSport0TxBuffer[0]) / 4),
												   (sSport0TxBuffer + sizeof(sSport0TxBuffer)/sizeof(sSport0TxBuffer[0]) * 3 / 4)
												 };

// semaphores and counters for reveived and transmitted blocks ( 1 block = 'Number_of_Samples' Frames )
volatile bool new_block_received = false;			// indicates that a full block has been received
volatile bool block_transmitted = false;			// indicates that a processed block has been transmitted

volatile int Number_of_received_blocks =0;			// counter of received blocks
volatile int Number_of_transmitted_blocks =0;		// counter of transmitted blocks

int interruptLatch;


// Operating states of channels
short Global_Status = PASS_THROUGH;
short Per_Channel_Status[Number_of_Channels];

short RIN_scaling_factor[Number_of_Channels];	// RIN scaling (number of bits to be shifted up)
short SIN_scaling_factor[Number_of_Channels];	// SIN scaling (number of bits to be shifted up)

#ifdef SOFTWARE_COMPANDING
	short Software_Companding = SOFTWARE_A_LAW;
#endif

// UART and Command Parser
Queue *cmdQueue;

// *****************************************************************************************************************************
// *****************************************************************************************************************************
// MAIN APPLICATION PROGRAM
// *****************************************************************************************************************************
// *****************************************************************************************************************************
section("L1_code") void main() 
{	int i;
#ifdef DUMP_VARIABLES
	char str[255];
#endif
			 
	// init exception handler
	Exception_Init();

	// Check Device
#ifndef	STANDALONE

#ifdef __ADSPBF533__
	printf("Compiled for ADSP-BF533 \n");
#endif

#ifdef __ADSPBF561__
	printf("Compiled for ADSP-BF561 \n");
#endif

#endif		// STANDALONE


	// set Clocks
	Set_PLL( (short)(CORECLK/CLKIN), (short)(CORECLK/SYSCLK));			// sets Core clock and SCLK

	// initialise SDRAM controller	
	InitSDRAM();

	// initialise ASYNC MEMORY controller	
	InitASYNC();
	
	// Init hardware (Flag IOs and COMETS)
	InitHardware();	

	// Init Trasmit Buffers
	for (i=0; i<sizeof(sSport0TxBuffer)/sizeof(sSport0TxBuffer[0]); i++)
			sSport0TxBuffer[i] = 0x0000;
	
	// initialise variables
	new_block_received = false;
	block_transmitted = false;

	Number_of_received_blocks =0;
	Number_of_transmitted_blocks =0;

	Global_Status = PASS_THROUGH;
	for (i=0; i<Number_of_Channels; i++)
	{	Per_Channel_Status[i] = ON;
		RIN_scaling_factor[i] = default_RIN_input_scaling;
		SIN_scaling_factor[i] = default_SIN_input_scaling;
	}
			
#ifdef SOFTWARE_COMPANDING
	Software_Companding = SOFTWARE_A_LAW;
#endif	
	
	// Configure UART comunication channel and Command Queue
	cmdQueue = createQueue();
	UART_Init();
		
#ifdef DUMP_VARIABLES
// Dump memory addresses to UART terminal for reference
	sprintf(str, "Global Enable at address 0x%x \n\r", &Global_Status);
	UART_puts(str);
#ifdef SOFTWARE_COMPANDING
	sprintf(str, "Select Software Companding Law at address 0x%x \n\r", &Software_Companding);
#endif	
	UART_puts(str);
	sprintf(str, "Per-Channel Enable at even addresses from 0x%x to 0x%x\n\r", Per_Channel_Status, Per_Channel_Status+Number_of_Channels-1);
	UART_puts(str);
	sprintf(str, "Per-Channel RIN scaling factors at even addresses from 0x%x to 0x%x\n\r", RIN_scaling_factor, RIN_scaling_factor+Number_of_Channels-1);
	UART_puts(str);
	sprintf(str, "Per-Channel SIN scaling factors at even addresses from 0x%x to 0x%x\n\r", SIN_scaling_factor, SIN_scaling_factor+Number_of_Channels-1);
	UART_puts(str);
	sprintf(str, "COMET1 base-address (RIN-SOUT): 0x%x \n\r", DEVICE1);
	UART_puts(str);
	sprintf(str, "COMET2 base-address (SIN-ROUT): 0x%x \n\r", DEVICE2);
	UART_puts(str);
#endif	// DUMP_VARIABLES

	UART_puts("Starting Command Parser\n\r");

	
	
	// Configure both COMET chips
	InitCOMET();
	
	// Configure Sport0 (RX,TX mulitchannel)  and DMA channels
	Sport0_Init();
	
	// init Interrupt handlers
	Interrupt_Init();

	
	// put COMET 2 in digital loopback mode (to measure bit rate error and delay)
	//	COMET_writeReg_verify(DEVICE2, 0x0A, 0x44);
	//	COMET_writeReg_verify(DEVICE1, 0x0A, 0x44);



	// start Data transfer
	UART_puts("Starting T1/E1 data transfers\n\r");
	Sport0_Enable();
	UART_puts("Ready.\n\r");
		
// *****************************************************************************************************************************
	// This main body loop assumes that the sin and rin buffers contain PCM sample values. The echo canceller outputs are in the SOUT buffers.
// *****************************************************************************************************************************
	while(1) {
		// if the queue is not empty, parse the commands
		if (getQueueCount(cmdQueue) > 0) {
			yyparse();
		}
		// Check for the semaphores, and process if both sin and rin were received
		if ((new_block_received == true) & (block_transmitted == true)) {
				asm("cli %0;" : "=d" (interruptLatch)); ssync();	// make non-interruptible
				new_block_received = false;					// reset the semaphores
				block_transmitted = false;
				asm("sti %0;" : : "d" (interruptLatch)); ssync();	// end of: make non-interruptible

				// do the processing of the block that was just received
				Process_Data();
				
		} 		// if ((new_block_received == true) & (block_transmitted == true))

	}			// while(1)

	
}				// main()

